'use strict';

var promises = require('fs/promises');
var lodash = require('lodash');
var vscode = require('vscode');
var vscodeWebviewShared = require('vscode-webview-shared');

class VSCodeWebview {
  baseUri;
  title;
  type;
  showOptions;
  options;
  panel;
  apiFactory;
  api;
  htmlPromise;
  html;
  index;
  apiPromise;
  constructor({
    path,
    index = "index.html",
    title,
    type = lodash.camelCase(title),
    showOptions = vscode.ViewColumn.One,
    context,
    apiFactory,
    options
  }) {
    this.baseUri = vscode.Uri.joinPath(context.extensionUri, path);
    this.title = title;
    this.type = type;
    this.showOptions = showOptions;
    this.apiFactory = apiFactory;
    const basePath = process.platform === "win32" ? this.baseUri.path.replace(/^\//, "") : this.baseUri.path;
    this.htmlPromise = promises.readFile(`${basePath}/${index}`, { encoding: "utf-8" });
    this.index = index;
    this.options = {
      enableScripts: true,
      localResourceRoots: [this.baseUri],
      ...options
    };
  }
  async buildHtml(baseSrc) {
    try {
      const htmlText = await this.htmlPromise;
      this.html = this.treatHTML(htmlText, baseSrc);
    } catch (error) {
      vscode.window.showErrorMessage("There was an error while loading the html for the webview. This is a bug, please report it to the team.");
      this.html = `
        <html>
          <body>
            <p>Unable to load webview from ${this.baseUri.path}/${this.index}.</p>
            <p>The underlying error is: ${vscodeWebviewShared.errorToString(error)}</p>
          </body>
        </html>
      `;
    }
    return this.html;
  }
  buildAPI(webview) {
    try {
      this.api = this.apiFactory?.call(null, webview);
      if (this.api && this.apiPromise) {
        this.apiPromise.resolve(this.api);
        this.apiPromise = void 0;
      }
    } catch (error) {
      vscode.window.showErrorMessage([
        "There was an error while building the webview: unable to instantiate the webview's API.",
        `This is a bug, please report it to the team. Cause: ${vscodeWebviewShared.errorToString(error)}`
      ].join("\n"));
    }
  }
  async buildPanel() {
    this.panel = vscode.window.createWebviewPanel(this.type, this.title, this.showOptions, this.options);
    this.buildAPI(this.panel.webview);
    const html = await this.buildHtml(this.panel?.webview.asWebviewUri(this.baseUri));
    this.panel.webview.html = html;
    this.panel.onDidDispose(() => {
      this.panel = void 0;
      this.api?.dispose();
      this.api = void 0;
    });
  }
  treatHTML(html, baseSrc) {
    return html.replace("<head>", `<head><base href="${baseSrc}/">`);
  }
  async show(column) {
    if (this.panel)
      this.panel.reveal(column);
    else
      await this.buildPanel();
  }
  getPanel() {
    return this.panel;
  }
  getHTML() {
    return this.html;
  }
  getAPI() {
    return this.api;
  }
  getAPIAsync() {
    if (this.api)
      return Promise.resolve(this.api);
    this.apiPromise = new vscodeWebviewShared.ManualPromise();
    return this.apiPromise.promise;
  }
}

class VSCodeViewProvider extends VSCodeWebview {
  view;
  constructor(options) {
    super(options);
    if (options.type) {
      options.context.subscriptions.push(vscode.window.registerWebviewViewProvider(options.type, this));
    }
  }
  async resolveWebviewView(view) {
    this.view = view;
    const { webview } = view;
    const html = this.getHTML() ?? await this.buildHtml(webview.asWebviewUri(this.baseUri));
    this.buildAPI(webview);
    webview.options = this.options;
    webview.html = html;
    view.onDidDispose(() => {
      this.view = void 0;
      this.api?.dispose();
      this.api = void 0;
    });
  }
  async show() {
    if (this.view)
      this.view.show();
    else
      vscode.window.showErrorMessage(
        "Can't show Stack Spot view because it has been disposed or not yet started. Please, restart the extension."
      );
  }
  getView() {
    return this.view;
  }
}

class MessageHandler {
  deps;
  getStateCalls = /* @__PURE__ */ new Map();
  setStateCalls = /* @__PURE__ */ new Map();
  constructor(deps) {
    this.deps = deps;
    this.listen();
  }
  async handleRequestToAPI(message) {
    vscodeWebviewShared.logger.debug("handling api call:", message);
    const { id, property = "", arguments: args = [] } = message;
    const fn = this.deps.getAPIHandler(property);
    if (!fn) {
      const error = `"${property}" is not a method or function of the api provided to the VSCodeWebview.`;
      this.deps.sendMessageToClient(vscodeWebviewShared.buildAPIError(id, error));
    }
    let functionHasExecuted = false;
    try {
      const result = await fn(...args);
      functionHasExecuted = true;
      vscodeWebviewShared.logger.debug("sending api response to client:", result);
      this.deps.sendMessageToClient(vscodeWebviewShared.buildAPIResponse(id, result));
    } catch (error) {
      const message2 = functionHasExecuted ? `Error while sending message to client. Please make sure the return value of the method "${property}" in the API provided to the VSCodeWebview is serializable.` : `Error while running method "${property}". Cause: ${vscodeWebviewShared.errorToString(error)}.`;
      this.deps.sendMessageToClient(vscodeWebviewShared.buildAPIError(id, message2));
      throw error;
    }
  }
  handleStateResponse(type, message) {
    vscodeWebviewShared.logger.debug(`handling ${type} state response:`, message);
    const map = type === "get" ? this.getStateCalls : this.setStateCalls;
    const promise = map.get(message.id);
    if (!promise) {
      const warning = type === "get" ? `received a state value from the client, but nothing awaits a state value in the backend. Message: ${message}.` : `finished setting a state on the client, but nothing awaits this operation in the backend. Message: ${message}`;
      return vscodeWebviewShared.logger.warn(warning);
    }
    if (message.error) {
      return promise.reject(message.error);
    }
    promise.resolve(message.result);
    map.delete(message.id);
  }
  listen() {
    this.deps.listenToMessagesFromClient(async (message) => {
      switch (message?.type) {
        case vscodeWebviewShared.messageType.api:
          this.handleRequestToAPI(message);
          break;
        case vscodeWebviewShared.messageType.getState:
          this.handleStateResponse("get", message);
          break;
        case vscodeWebviewShared.messageType.setState:
          this.handleStateResponse("set", message);
      }
    });
  }
  dispose() {
    this.getStateCalls.forEach((value, key) => {
      value.reject(`The webview closed before the state "${String(key)}" could be retrieved.`);
    });
    this.setStateCalls.forEach((value, key) => {
      value.reject(`The webview closed before the state "${key}" could be set.`);
    });
    this.getStateCalls.clear();
    this.setStateCalls.clear();
  }
  getState(name) {
    vscodeWebviewShared.logger.debug("getting state from client:", name);
    const state = String(name);
    const message = vscodeWebviewShared.buildGetStateRequest(state);
    let manualPromise = this.getStateCalls.get(message.id);
    if (!manualPromise) {
      manualPromise = new vscodeWebviewShared.ManualPromise();
      this.getStateCalls.set(message.id, manualPromise);
      vscodeWebviewShared.logger.debug("sending get state message to client:", message);
      this.deps.sendMessageToClient(message);
    }
    return manualPromise.promise;
  }
  setState(name, value) {
    vscodeWebviewShared.logger.debug("setting state in client:", name, value);
    const state = String(name);
    const manualPromise = new vscodeWebviewShared.ManualPromise();
    try {
      const message = vscodeWebviewShared.buildSetStateRequest(state, value);
      this.setStateCalls.set(message.id, manualPromise);
      vscodeWebviewShared.logger.debug("sending set state message to client:", message);
      this.deps.sendMessageToClient(message);
      return manualPromise.promise;
    } catch (error) {
      manualPromise.reject(`Can't set state with name "${state}". Please, make sure the value passed as parameter is serializable.`);
      throw error;
    }
  }
}

class VSCodeWebviewAPI {
  #messageHandler;
  state;
  constructor(webview) {
    this.#messageHandler = this.#createMessageHandler(webview);
    this.state = {
      get: (key) => this.#messageHandler.getState.apply(this.#messageHandler, [String(key)]),
      set: (key, value) => this.#messageHandler.setState.apply(this.#messageHandler, [String(key), value])
    };
  }
  #createMessageHandler(webview) {
    return new MessageHandler({
      getAPIHandler: (name) => {
        const member = this[name];
        return typeof member === "function" ? (...args) => member.apply(this, args) : void 0;
      },
      sendMessageToClient: (message) => webview.postMessage.apply(webview, [message]),
      listenToMessagesFromClient: (listener) => {
        webview.onDidReceiveMessage((data) => {
          vscodeWebviewShared.logger.debug("received message from client:", data);
          const message = vscodeWebviewShared.asWebViewMessage(data);
          if (message)
            listener(message);
        });
      }
    });
  }
  dispose() {
    this.#messageHandler.dispose();
  }
}

Object.defineProperty(exports, 'AsyncState', {
  enumerable: true,
  get: function () { return vscodeWebviewShared.AsyncState; }
});
Object.defineProperty(exports, 'AsyncStateful', {
  enumerable: true,
  get: function () { return vscodeWebviewShared.AsyncStateful; }
});
Object.defineProperty(exports, 'ManualPromise', {
  enumerable: true,
  get: function () { return vscodeWebviewShared.ManualPromise; }
});
Object.defineProperty(exports, 'errorToString', {
  enumerable: true,
  get: function () { return vscodeWebviewShared.errorToString; }
});
exports.VSCodeViewProvider = VSCodeViewProvider;
exports.VSCodeWebview = VSCodeWebview;
exports.VSCodeWebviewAPI = VSCodeWebviewAPI;
//# sourceMappingURL=index.js.map
